@@ -8,7 +8,7 @@
 #![allow(clippy::exhaustive_structs)]
 
 use std::{
-    collections::{BTreeMap, BTreeSet},
+    collections::{HashMap, HashSet},
     convert::{TryFrom, TryInto},
     sync::{
         atomic::{AtomicU64, Ordering::SeqCst},
@@ -19,7 +19,7 @@ use std::{
 use criterion::{criterion_group, criterion_main, Criterion};
 use event::StateEvent;
 use js_int::uint;
-use maplit::{btreemap, btreeset};
+use maplit::{btreemap, hashmap, hashset};
 use ruma_common::MilliSecondsSinceUnixEpoch;
 use ruma_events::{
     pdu::{EventHash, Pdu, RoomV3Pdu},
@@ -37,16 +37,16 @@ static SERVER_TIMESTAMP: AtomicU64 = AtomicU64::new(0);
 
 fn lexico_topo_sort(c: &mut Criterion) {
     c.bench_function("lexicographical topological sort", |b| {
-        let graph = btreemap! {
-            event_id("l") => btreeset![event_id("o")],
-            event_id("m") => btreeset![event_id("n"), event_id("o")],
-            event_id("n") => btreeset![event_id("o")],
-            event_id("o") => btreeset![], // "o" has zero outgoing edges but 4 incoming edges
-            event_id("p") => btreeset![event_id("o")],
+        let graph = hashmap! {
+            event_id("l") => hashset![event_id("o")],
+            event_id("m") => hashset![event_id("n"), event_id("o")],
+            event_id("n") => hashset![event_id("o")],
+            event_id("o") => hashset![], // "o" has zero outgoing edges but 4 incoming edges
+            event_id("p") => hashset![event_id("o")],
         };
         b.iter(|| {
             let _ = StateResolution::lexicographical_topological_sort(&graph, |id| {
-                (0, MilliSecondsSinceUnixEpoch(uint!(0)), id.clone())
+                Ok((0, MilliSecondsSinceUnixEpoch(uint!(0)), id.clone()))
             });
         })
     });
@@ -54,15 +54,15 @@ fn lexico_topo_sort(c: &mut Criterion) {
 
 fn resolution_shallow_auth_chain(c: &mut Criterion) {
     c.bench_function("resolve state of 5 events one fork", |b| {
-        let mut store = TestStore(btreemap! {});
+        let mut store = TestStore(hashmap! {});
 
         // build up the DAG
         let (state_at_bob, state_at_charlie, _) = store.set_up();
 
         b.iter(|| {
-            let mut ev_map: EventMap<Arc<StateEvent>> = store.0.clone();
+            let ev_map: EventMap<Arc<StateEvent>> = store.0.clone();
             let state_sets = vec![state_at_bob.clone(), state_at_charlie.clone()];
-            let _ = match StateResolution::resolve::<StateEvent>(
+            let _ = match StateResolution::resolve::<StateEvent, _>(
                 &room_id(),
                 &RoomVersionId::Version6,
                 &state_sets,
@@ -74,7 +74,7 @@ fn resolution_shallow_auth_chain(c: &mut Criterion) {
                             .unwrap()
                     })
                     .collect(),
-                &mut ev_map,
+                |id| ev_map.get(id).map(Arc::clone),
             ) {
                 Ok(state) => state,
                 Err(e) => panic!("{}", e),
@@ -119,7 +119,7 @@ fn resolve_deeper_event_set(c: &mut Criterion) {
 
         b.iter(|| {
             let state_sets = vec![state_set_a.clone(), state_set_b.clone()];
-            let _ = match StateResolution::resolve::<StateEvent>(
+            let _ = match StateResolution::resolve::<StateEvent, _>(
                 &room_id(),
                 &RoomVersionId::Version6,
                 &state_sets,
@@ -131,7 +131,7 @@ fn resolve_deeper_event_set(c: &mut Criterion) {
                             .unwrap()
                     })
                     .collect(),
-                &mut inner,
+                |id| inner.get(id).map(Arc::clone),
             ) {
                 Ok(state) => state,
                 Err(_) => panic!("resolution failed during benchmarking"),
@@ -154,7 +154,7 @@ criterion_main!(benches);
 //  IMPLEMENTATION DETAILS AHEAD
 //
 /////////////////////////////////////////////////////////////////////*/
-pub struct TestStore<E: Event>(pub BTreeMap<EventId, Arc<E>>);
+pub struct TestStore<E: Event>(pub HashMap<EventId, Arc<E>>);
 
 #[allow(unused)]
 impl<E: Event> TestStore<E> {
@@ -175,8 +175,12 @@ impl<E: Event> TestStore<E> {
     }
 
     /// Returns a Vec of the related auth events to the given `event`.
-    pub fn auth_event_ids(&self, room_id: &RoomId, event_ids: &[EventId]) -> Result<Vec<EventId>> {
-        let mut result = vec![];
+    pub fn auth_event_ids(
+        &self,
+        room_id: &RoomId,
+        event_ids: &[EventId],
+    ) -> Result<HashSet<EventId>> {
+        let mut result = HashSet::new();
         let mut stack = event_ids.to_vec();
 
         // DFS for auth event chain
@@ -186,7 +190,7 @@ impl<E: Event> TestStore<E> {
                 continue;
             }
 
-            result.push(ev_id.clone());
+            result.insert(ev_id.clone());
 
             let event = self.get_event(room_id, &ev_id)?;
 
@@ -202,26 +206,21 @@ impl<E: Event> TestStore<E> {
         room_id: &RoomId,
         event_ids: Vec<Vec<EventId>>,
     ) -> Result<Vec<EventId>> {
-        let mut chains = vec![];
+        let mut auth_chain_sets = vec![];
         for ids in event_ids {
             // TODO state store `auth_event_ids` returns self in the event ids list
             // when an event returns `auth_event_ids` self is not contained
-            let chain = self.auth_event_ids(room_id, &ids)?.into_iter().collect::<BTreeSet<_>>();
-            chains.push(chain);
+            let chain = self.auth_event_ids(room_id, &ids)?.into_iter().collect::<HashSet<_>>();
+            auth_chain_sets.push(chain);
         }
 
-        if let Some(chain) = chains.first() {
-            let rest = chains.iter().skip(1).flatten().cloned().collect();
-            let common = chain.intersection(&rest).collect::<Vec<_>>();
-
-            Ok(chains
+        if let Some(first) = auth_chain_sets.first().cloned() {
+            let common = auth_chain_sets
                 .iter()
-                .flatten()
-                .filter(|id| !common.contains(id))
-                .cloned()
-                .collect::<BTreeSet<_>>()
-                .into_iter()
-                .collect())
+                .skip(1)
+                .fold(first, |a, b| a.intersection(b).cloned().collect::<HashSet<EventId>>());
+
+            Ok(auth_chain_sets.into_iter().flatten().filter(|id| !common.contains(id)).collect())
         } else {
             Ok(vec![])
         }
@@ -354,11 +353,11 @@ where
     // We don't care if the addition happens in order just that it is atomic
     // (each event has its own value)
     let ts = SERVER_TIMESTAMP.fetch_add(1, SeqCst);
-    let id = if id.contains('$') { id.to_string() } else { format!("${}:foo", id) };
+    let id = if id.contains('$') { id.to_owned() } else { format!("${}:foo", id) };
     let auth_events = auth_events.iter().map(AsRef::as_ref).map(event_id).collect::<Vec<_>>();
     let prev_events = prev_events.iter().map(AsRef::as_ref).map(event_id).collect::<Vec<_>>();
 
-    let state_key = state_key.map(ToString::to_string);
+    let state_key = state_key.map(ToOwned::to_owned);
     Arc::new(StateEvent {
         event_id: EventId::try_from(id).unwrap(),
         rest: Pdu::RoomV3Pdu(RoomV3Pdu {
@@ -383,7 +382,7 @@ where
 
 // all graphs start with these input events
 #[allow(non_snake_case)]
-fn INITIAL_EVENTS() -> BTreeMap<EventId, Arc<StateEvent>> {
+fn INITIAL_EVENTS() -> HashMap<EventId, Arc<StateEvent>> {
     vec![
         to_pdu_event::<EventId>(
             "CREATE",
@@ -465,7 +464,7 @@ fn INITIAL_EVENTS() -> BTreeMap<EventId, Arc<StateEvent>> {
 
 // all graphs start with these input events
 #[allow(non_snake_case)]
-fn BAN_STATE_SET() -> BTreeMap<EventId, Arc<StateEvent>> {
+fn BAN_STATE_SET() -> HashMap<EventId, Arc<StateEvent>> {
     vec![
         to_pdu_event(
             "PA",
