@@ -1,10 +1,11 @@
 use std::{
     cmp::Reverse,
-    collections::{BTreeMap, BTreeSet, BinaryHeap},
+    collections::{BinaryHeap, HashMap, HashSet},
     sync::Arc,
 };
 
-use maplit::btreeset;
+use itertools::Itertools;
+use maplit::hashset;
 use ruma_common::MilliSecondsSinceUnixEpoch;
 use ruma_events::{
     room::{
@@ -14,7 +15,7 @@ use ruma_events::{
     EventType,
 };
 use ruma_identifiers::{EventId, RoomId, RoomVersionId};
-use tracing::{debug, info, warn};
+use tracing::{debug, info, trace, warn};
 
 mod error;
 pub mod event_auth;
@@ -27,10 +28,10 @@ pub use room_version::RoomVersion;
 pub use state_event::Event;
 
 /// A mapping of event type and state_key to some value `T`, usually an `EventId`.
-pub type StateMap<T> = BTreeMap<(EventType, String), T>;
+pub type StateMap<T> = HashMap<(EventType, String), T>;
 
 /// A mapping of `EventId` to `T`, usually a `ServerPdu`.
-pub type EventMap<T> = BTreeMap<EventId, T>;
+pub type EventMap<T> = HashMap<EventId, T>;
 
 #[derive(Default)]
 #[allow(clippy::exhaustive_structs)]
@@ -45,39 +46,52 @@ impl StateResolution {
     /// * `state_sets` - The incoming state to resolve. Each `StateMap` represents a possible fork
     /// in the state of a room.
     ///
-    /// * `auth_events` - The full recursive set of `auth_events` for each event in the
+    /// * `auth_chain_sets` - The full recursive set of `auth_events` for each event in the
     ///   `state_sets`.
     ///
-    /// * `event_map` - The `EventMap` acts as a local cache of state, any event that is not found
-    /// in the `event_map` will cause an unrecoverable `Error` in `resolve`.
-    pub fn resolve<E: Event>(
+    /// * `fetch_event` - Any event not found in the `event_map` will defer to this closure to find
+    /// the event.
+    ///
+    /// ## Invariants
+    ///
+    /// The caller of `StateResolution::resolve` must ensure that all the events are from the same
+    /// room. Although this function takes a `RoomId` it does not check that each event is part
+    /// of the same room.
+    pub fn resolve<E, F>(
         room_id: &RoomId,
         room_version: &RoomVersionId,
         state_sets: &[StateMap<EventId>],
-        auth_events: Vec<Vec<EventId>>,
-        event_map: &mut EventMap<Arc<E>>,
-    ) -> Result<StateMap<EventId>> {
+        auth_chain_sets: Vec<HashSet<EventId>>,
+        fetch_event: F,
+    ) -> Result<StateMap<EventId>>
+    where
+        E: Event,
+        F: Fn(&EventId) -> Option<Arc<E>>,
+    {
         info!("State resolution starting");
 
         // Split non-conflicting and conflicting state
         let (clean, conflicting) = StateResolution::separate(state_sets);
 
-        info!("non conflicting {:?}", clean.len());
+        info!("non conflicting events: {}", clean.len());
+        trace!("{:?}", clean);
 
         if conflicting.is_empty() {
             info!("no conflicting state found");
             return Ok(clean);
         }
 
-        info!("{} conflicting events", conflicting.len());
+        info!("conflicting events: {}", conflicting.len());
+        debug!("{:?}", conflicting);
 
         // The set of auth events that are not common across server forks
-        let mut auth_diff = StateResolution::get_auth_chain_diff(room_id, &auth_events)?;
-
-        debug!("auth diff size {:?}", auth_diff);
+        let mut auth_diff = StateResolution::get_auth_chain_diff(room_id, auth_chain_sets)?;
 
         // Add the auth_diff to conflicting now we have a full set of conflicting events
-        auth_diff.extend(conflicting.values().cloned().flatten());
+        auth_diff.extend(conflicting.values().cloned().flatten().flatten());
+
+        debug!("auth diff: {}", auth_diff.len());
+        trace!("{:?}", auth_diff);
 
         // `all_conflicted` contains unique items
         // synapse says `full_set = {eid for eid in full_conflicted_set if eid in event_map}`
@@ -85,9 +99,10 @@ impl StateResolution {
         // Don't honor events we cannot "verify"
         // TODO: BTreeSet::retain() when stable 1.53
         let all_conflicted =
-            auth_diff.into_iter().filter(|id| event_map.contains_key(id)).collect::<BTreeSet<_>>();
+            auth_diff.into_iter().filter(|id| fetch_event(id).is_some()).collect::<HashSet<_>>();
 
-        info!("full conflicted set is {} events", all_conflicted.len());
+        info!("full conflicted set: {}", all_conflicted.len());
+        debug!("{:?}", all_conflicted);
 
         // We used to check that all events are events from the correct room
         // this is now a check the caller of `resolve` must make.
@@ -95,35 +110,35 @@ impl StateResolution {
         // Get only the control events with a state_key: "" or ban/kick event (sender != state_key)
         let control_events = all_conflicted
             .iter()
-            .filter(|id| is_power_event_id(id, event_map))
+            .filter(|id| is_power_event_id(id, &fetch_event))
             .cloned()
             .collect::<Vec<_>>();
 
         // Sort the control events based on power_level/clock/event_id and outgoing/incoming edges
         let sorted_control_levels = StateResolution::reverse_topological_power_sort(
-            room_id,
             &control_events,
-            event_map,
             &all_conflicted,
-        );
+            &fetch_event,
+        )?;
 
-        debug!("SRTD {:?}", sorted_control_levels);
+        debug!("sorted control events: {}", sorted_control_levels.len());
+        trace!("{:?}", sorted_control_levels);
 
         let room_version = RoomVersion::new(room_version)?;
         // Sequentially auth check each control event.
         let resolved_control = StateResolution::iterative_auth_check(
-            room_id,
             &room_version,
             &sorted_control_levels,
             &clean,
-            event_map,
+            &fetch_event,
         )?;
 
-        debug!("AUTHED {:?}", resolved_control.iter().collect::<Vec<_>>());
+        debug!("resolved control events: {}", resolved_control.len());
+        trace!("{:?}", resolved_control);
 
         // At this point the control_events have been resolved we now have to
         // sort the remaining events using the mainline of the resolved power level.
-        let deduped_power_ev = sorted_control_levels.into_iter().collect::<BTreeSet<_>>();
+        let deduped_power_ev = sorted_control_levels.into_iter().collect::<HashSet<_>>();
 
         // This removes the control events that passed auth and more importantly those that failed
         // auth
@@ -133,24 +148,24 @@ impl StateResolution {
             .cloned()
             .collect::<Vec<_>>();
 
-        debug!("LEFT {:?}", events_to_resolve.iter().collect::<Vec<_>>());
+        debug!("events left to resolve: {}", events_to_resolve.len());
+        trace!("{:?}", events_to_resolve);
 
         // This "epochs" power level event
         let power_event = resolved_control.get(&(EventType::RoomPowerLevels, "".into()));
 
-        debug!("PL {:?}", power_event);
+        debug!("power event: {:?}", power_event);
 
         let sorted_left_events =
-            StateResolution::mainline_sort(room_id, &events_to_resolve, power_event, event_map);
+            StateResolution::mainline_sort(&events_to_resolve, power_event, &fetch_event)?;
 
-        debug!("SORTED LEFT {:?}", sorted_left_events.iter().collect::<Vec<_>>());
+        trace!("events left, sorted: {:?}", sorted_left_events.iter().collect::<Vec<_>>());
 
         let mut resolved_state = StateResolution::iterative_auth_check(
-            room_id,
             &room_version,
             &sorted_left_events,
             &resolved_control, // The control events are added to the final resolved state
-            event_map,
+            &fetch_event,
         )?;
 
         // Add unconflicted state to the resolved state
@@ -167,9 +182,7 @@ impl StateResolution {
     /// that none of the other have this is a conflicting event.
     pub fn separate(
         state_sets: &[StateMap<EventId>],
-    ) -> (StateMap<EventId>, StateMap<Vec<EventId>>) {
-        use itertools::Itertools;
-
+    ) -> (StateMap<EventId>, StateMap<Vec<Option<EventId>>>) {
         info!("separating {} sets of events into conflicted/unconflicted", state_sets.len());
 
         let mut unconflicted_state = StateMap::new();
@@ -177,16 +190,14 @@ impl StateResolution {
 
         for key in state_sets.iter().flat_map(|map| map.keys()).unique() {
             let mut event_ids =
-                state_sets.iter().map(|state_set| state_set.get(key)).unique().collect::<Vec<_>>();
+                state_sets.iter().map(|state_set| state_set.get(key)).collect::<Vec<_>>();
 
-            if event_ids.len() == 1 {
+            if event_ids.iter().all_equal() {
                 let id = event_ids.remove(0).expect("unconflicting `EventId` is not None");
                 unconflicted_state.insert(key.clone(), id.clone());
             } else {
-                conflicted_state.insert(
-                    key.clone(),
-                    event_ids.into_iter().flatten().cloned().collect::<Vec<_>>(),
-                );
+                conflicted_state
+                    .insert(key.clone(), event_ids.into_iter().map(|o| o.cloned()).collect());
             }
         }
 
@@ -196,24 +207,17 @@ impl StateResolution {
     /// Returns a Vec of deduped EventIds that appear in some chains but not others.
     pub fn get_auth_chain_diff(
         _room_id: &RoomId,
-        auth_event_ids: &[Vec<EventId>],
-    ) -> Result<BTreeSet<EventId>> {
-        let mut chains = vec![];
+        auth_chain_sets: Vec<HashSet<EventId>>,
+    ) -> Result<HashSet<EventId>> {
+        if let Some(first) = auth_chain_sets.first().cloned() {
+            let common = auth_chain_sets
+                .iter()
+                .skip(1)
+                .fold(first, |a, b| a.intersection(b).cloned().collect::<HashSet<EventId>>());
 
-        for ids in auth_event_ids {
-            // TODO state store `auth_event_ids` returns self in the event ids list
-            // when an event returns `auth_event_ids` self is not contained
-            let chain = ids.iter().cloned().collect::<BTreeSet<_>>();
-            chains.push(chain);
-        }
-
-        if let Some(chain) = chains.first().cloned() {
-            let rest = chains.iter().skip(1).flatten().cloned().collect();
-            let common = chain.intersection(&rest).collect::<Vec<_>>();
-
-            Ok(chains.into_iter().flatten().filter(|id| !common.contains(&id)).collect())
+            Ok(auth_chain_sets.into_iter().flatten().filter(|id| !common.contains(id)).collect())
         } else {
-            Ok(btreeset![])
+            Ok(hashset![])
         }
     }
 
@@ -224,18 +228,24 @@ impl StateResolution {
     ///
     /// The power level is negative because a higher power level is equated to an
     /// earlier (further back in time) origin server timestamp.
-    pub fn reverse_topological_power_sort<E: Event>(
-        room_id: &RoomId,
+    pub fn reverse_topological_power_sort<E, F>(
         events_to_sort: &[EventId],
-        event_map: &mut EventMap<Arc<E>>,
-        auth_diff: &BTreeSet<EventId>,
-    ) -> Vec<EventId> {
+        auth_diff: &HashSet<EventId>,
+        fetch_event: F,
+    ) -> Result<Vec<EventId>>
+    where
+        E: Event,
+        F: Fn(&EventId) -> Option<Arc<E>>,
+    {
         debug!("reverse topological sort of power events");
 
-        let mut graph = BTreeMap::new();
+        let mut graph = HashMap::new();
         for event_id in events_to_sort.iter() {
             StateResolution::add_event_and_auth_chain_to_graph(
-                room_id, &mut graph, event_id, event_map, auth_diff,
+                &mut graph,
+                event_id,
+                auth_diff,
+                &fetch_event,
             );
 
             // TODO: if these functions are ever made async here
@@ -244,9 +254,9 @@ impl StateResolution {
         }
 
         // This is used in the `key_fn` passed to the lexico_topo_sort fn
-        let mut event_to_pl = BTreeMap::new();
+        let mut event_to_pl = HashMap::new();
         for event_id in graph.keys() {
-            let pl = StateResolution::get_power_level_for_sender(room_id, event_id, event_map);
+            let pl = StateResolution::get_power_level_for_sender(event_id, &fetch_event);
             info!("{} power level {}", event_id, pl);
 
             event_to_pl.insert(event_id.clone(), pl);
@@ -257,15 +267,15 @@ impl StateResolution {
         }
 
         StateResolution::lexicographical_topological_sort(&graph, |event_id| {
-            let ev = event_map.get(event_id).unwrap();
-            let pl = event_to_pl.get(event_id).unwrap();
+            let ev = fetch_event(event_id).ok_or_else(|| Error::NotFound("".into()))?;
+            let pl = event_to_pl.get(event_id).ok_or_else(|| Error::NotFound("".into()))?;
 
             debug!("{:?}", (-*pl, ev.origin_server_ts(), &ev.event_id()));
 
             // This return value is the key used for sorting events,
             // events are then sorted by power level, time,
             // and lexically by event_id.
-            (-*pl, ev.origin_server_ts(), ev.event_id().clone())
+            Ok((-*pl, ev.origin_server_ts(), ev.event_id().clone()))
         })
     }
 
@@ -273,11 +283,11 @@ impl StateResolution {
     /// `key_fn` is used as a tie breaker. The tie breaker happens based on
     /// power level, age, and event_id.
     pub fn lexicographical_topological_sort<F>(
-        graph: &BTreeMap<EventId, BTreeSet<EventId>>,
+        graph: &HashMap<EventId, HashSet<EventId>>,
         key_fn: F,
-    ) -> Vec<EventId>
+    ) -> Result<Vec<EventId>>
     where
-        F: Fn(&EventId) -> (i64, MilliSecondsSinceUnixEpoch, EventId),
+        F: Fn(&EventId) -> Result<(i64, MilliSecondsSinceUnixEpoch, EventId)>,
     {
         info!("starting lexicographical topological sort");
         // NOTE: an event that has no incoming edges happened most recently,
@@ -287,13 +297,13 @@ impl StateResolution {
         // outgoing edges, c.f.
         // https://en.wikipedia.org/wiki/Topological_sorting#Kahn's_algorithm
 
-        // TODO make the BTreeSet conversion cleaner ??
         // outdegree_map is an event referring to the events before it, the
         // more outdegree's the more recent the event.
         let mut outdegree_map = graph.clone();
 
-        // The number of events that depend on the given event (the eventId key)
-        let mut reverse_graph = BTreeMap::new();
+        // The number of events that depend on the given event (the EventId key)
+        // How many events reference this event in the DAG as a parent
+        let mut reverse_graph = HashMap::new();
 
         // Vec of nodes that have zero out degree, least recent events.
         let mut zero_outdegree = vec![];
@@ -302,12 +312,12 @@ impl StateResolution {
             if edges.is_empty() {
                 // The `Reverse` is because rusts `BinaryHeap` sorts largest -> smallest we need
                 // smallest -> largest
-                zero_outdegree.push(Reverse((key_fn(node), node)));
+                zero_outdegree.push(Reverse((key_fn(node)?, node)));
             }
 
-            reverse_graph.entry(node).or_insert(btreeset![]);
+            reverse_graph.entry(node).or_insert(hashset![]);
             for edge in edges {
-                reverse_graph.entry(edge).or_insert(btreeset![]).insert(node);
+                reverse_graph.entry(edge).or_insert(hashset![]).insert(node);
             }
         }
 
@@ -318,14 +328,17 @@ impl StateResolution {
         // Destructure the `Reverse` and take the smallest `node` each time
         while let Some(Reverse((_, node))) = heap.pop() {
             let node: &EventId = node;
-            for parent in reverse_graph.get(node).unwrap() {
+            for parent in reverse_graph.get(node).expect("EventId in heap is also in reverse_graph")
+            {
                 // The number of outgoing edges this node has
-                let out = outdegree_map.get_mut(parent).unwrap();
+                let out = outdegree_map
+                    .get_mut(parent)
+                    .expect("outdegree_map knows of all referenced EventIds");
 
                 // Only push on the heap once older events have been cleared
                 out.remove(node);
                 if out.is_empty() {
-                    heap.push(Reverse((key_fn(parent), parent)));
+                    heap.push(Reverse((key_fn(parent)?, parent)));
                 }
             }
 
@@ -333,24 +346,22 @@ impl StateResolution {
             sorted.push(node.clone());
         }
 
-        sorted
+        Ok(sorted)
     }
 
     /// Find the power level for the sender of `event_id` or return a default value of zero.
-    fn get_power_level_for_sender<E: Event>(
-        room_id: &RoomId,
-        event_id: &EventId,
-        event_map: &mut EventMap<Arc<E>>,
-    ) -> i64 {
+    fn get_power_level_for_sender<E, F>(event_id: &EventId, fetch_event: F) -> i64
+    where
+        E: Event,
+        F: Fn(&EventId) -> Option<Arc<E>>,
+    {
         info!("fetch event ({}) senders power level", event_id);
 
-        let event = StateResolution::get_or_load_event(room_id, event_id, event_map);
+        let event = fetch_event(event_id);
         let mut pl = None;
 
-        // TODO store.auth_event_ids returns "self" with the event ids is this ok
-        // event.auth_event_ids does not include its own event id ?
         for aid in event.as_ref().map(|pdu| pdu.auth_events()).unwrap_or_default() {
-            if let Ok(aev) = StateResolution::get_or_load_event(room_id, &aid, event_map) {
+            if let Some(aev) = fetch_event(&aid) {
                 if is_type_and_key(&aev, EventType::RoomPowerLevels, "") {
                     pl = Some(aev);
                     break;
@@ -365,7 +376,7 @@ impl StateResolution {
         if let Some(content) =
             pl.and_then(|pl| serde_json::from_value::<PowerLevelsEventContent>(pl.content()).ok())
         {
-            if let Ok(ev) = event {
+            if let Some(ev) = event {
                 if let Some(user) = content.users.get(ev.sender()) {
                     debug!("found {} at power_level {}", ev.sender(), user);
                     return (*user).into();
@@ -387,13 +398,16 @@ impl StateResolution {
     /// For each `events_to_check` event we gather the events needed to auth it from the
     /// `event_map` or `store` and verify each event using the `event_auth::auth_check`
     /// function.
-    pub fn iterative_auth_check<E: Event>(
-        room_id: &RoomId,
+    pub fn iterative_auth_check<E, F>(
         room_version: &RoomVersion,
         events_to_check: &[EventId],
         unconflicted_state: &StateMap<EventId>,
-        event_map: &mut EventMap<Arc<E>>,
-    ) -> Result<StateMap<EventId>> {
+        fetch_event: F,
+    ) -> Result<StateMap<EventId>>
+    where
+        E: Event,
+        F: Fn(&EventId) -> Option<Arc<E>>,
+    {
         info!("starting iterative auth check");
 
         debug!("performing auth checks on {:?}", events_to_check.iter().collect::<Vec<_>>());
@@ -401,14 +415,15 @@ impl StateResolution {
         let mut resolved_state = unconflicted_state.clone();
 
         for event_id in events_to_check.iter() {
-            let event = StateResolution::get_or_load_event(room_id, event_id, event_map)?;
+            let event = fetch_event(event_id)
+                .ok_or_else(|| Error::NotFound(format!("Failed to find {}", event_id)))?;
             let state_key = event
                 .state_key()
                 .ok_or_else(|| Error::InvalidPdu("State event had no state key".to_owned()))?;
 
-            let mut auth_events = BTreeMap::new();
+            let mut auth_events = HashMap::new();
             for aid in &event.auth_events() {
-                if let Ok(ev) = StateResolution::get_or_load_event(room_id, aid, event_map) {
+                if let Some(ev) = fetch_event(aid) {
                     // TODO synapse check "rejected_reason", I'm guessing this is redacted_because
                     // in ruma ??
                     auth_events.insert(
@@ -432,8 +447,7 @@ impl StateResolution {
                 event.content(),
             ) {
                 if let Some(ev_id) = resolved_state.get(&key) {
-                    if let Ok(event) = StateResolution::get_or_load_event(room_id, ev_id, event_map)
-                    {
+                    if let Some(event) = fetch_event(ev_id) {
                         // TODO synapse checks `rejected_reason` is None here
                         auth_events.insert(key.clone(), event);
                     }
@@ -442,11 +456,8 @@ impl StateResolution {
 
             debug!("event to check {:?}", event.event_id());
 
-            let most_recent_prev_event = event
-                .prev_events()
-                .iter()
-                .filter_map(|id| StateResolution::get_or_load_event(room_id, id, event_map).ok())
-                .next_back();
+            let most_recent_prev_event =
+                event.prev_events().iter().filter_map(|id| fetch_event(id)).next_back();
 
             // The key for this is (eventType + a state_key of the signed token not sender) so
             // search for it
@@ -485,17 +496,20 @@ impl StateResolution {
     /// power_level event. If there have been two power events the after the most recent are
     /// depth 0, the events before (with the first power level as a parent) will be marked
     /// as depth 1. depth 1 is "older" than depth 0.
-    pub fn mainline_sort<E: Event>(
-        room_id: &RoomId,
+    pub fn mainline_sort<E, F>(
         to_sort: &[EventId],
         resolved_power_level: Option<&EventId>,
-        event_map: &mut EventMap<Arc<E>>,
-    ) -> Vec<EventId> {
+        fetch_event: F,
+    ) -> Result<Vec<EventId>>
+    where
+        E: Event,
+        F: Fn(&EventId) -> Option<Arc<E>>,
+    {
         debug!("mainline sort of events");
 
         // There are no EventId's to sort, bail.
         if to_sort.is_empty() {
-            return vec![];
+            return Ok(vec![]);
         }
 
         let mut mainline = vec![];
@@ -503,11 +517,13 @@ impl StateResolution {
         while let Some(p) = pl {
             mainline.push(p.clone());
 
-            let event = StateResolution::get_or_load_event(room_id, &p, event_map).unwrap();
+            let event =
+                fetch_event(&p).ok_or_else(|| Error::NotFound(format!("Failed to find {}", p)))?;
             let auth_events = &event.auth_events();
             pl = None;
             for aid in auth_events {
-                let ev = StateResolution::get_or_load_event(room_id, aid, event_map).unwrap();
+                let ev = fetch_event(aid)
+                    .ok_or_else(|| Error::NotFound(format!("Failed to find {}", aid)))?;
                 if is_type_and_key(&ev, EventType::RoomPowerLevels, "") {
                     pl = Some(aid.clone());
                     break;
@@ -523,22 +539,19 @@ impl StateResolution {
             .rev()
             .enumerate()
             .map(|(idx, eid)| ((*eid).clone(), idx))
-            .collect::<BTreeMap<_, _>>();
+            .collect::<HashMap<_, _>>();
 
-        let mut order_map = BTreeMap::new();
+        let mut order_map = HashMap::new();
         for ev_id in to_sort.iter() {
-            if let Ok(event) = StateResolution::get_or_load_event(room_id, ev_id, event_map) {
-                if let Ok(depth) = StateResolution::get_mainline_depth(
-                    room_id,
-                    Some(event),
-                    &mainline_map,
-                    event_map,
-                ) {
+            if let Some(event) = fetch_event(ev_id) {
+                if let Ok(depth) =
+                    StateResolution::get_mainline_depth(Some(event), &mainline_map, &fetch_event)
+                {
                     order_map.insert(
                         ev_id,
                         (
                             depth,
-                            event_map.get(ev_id).map(|ev| ev.origin_server_ts()),
+                            fetch_event(ev_id).map(|ev| ev.origin_server_ts()),
                             ev_id, // TODO should this be a &str to sort lexically??
                         ),
                     );
@@ -555,17 +568,20 @@ impl StateResolution {
         let mut sort_event_ids = order_map.keys().map(|&k| k.clone()).collect::<Vec<_>>();
         sort_event_ids.sort_by_key(|sort_id| order_map.get(sort_id).unwrap());
 
-        sort_event_ids
+        Ok(sort_event_ids)
     }
 
     /// Get the mainline depth from the `mainline_map` or finds a power_level event
     /// that has an associated mainline depth.
-    fn get_mainline_depth<E: Event>(
-        room_id: &RoomId,
+    fn get_mainline_depth<E, F>(
         mut event: Option<Arc<E>>,
         mainline_map: &EventMap<usize>,
-        event_map: &mut EventMap<Arc<E>>,
-    ) -> Result<usize> {
+        fetch_event: F,
+    ) -> Result<usize>
+    where
+        E: Event,
+        F: Fn(&EventId) -> Option<Arc<E>>,
+    {
         while let Some(sort_ev) = event {
             debug!("mainline event_id {}", sort_ev.event_id());
             let id = &sort_ev.event_id();
@@ -576,7 +592,8 @@ impl StateResolution {
             let auth_events = &sort_ev.auth_events();
             event = None;
             for aid in auth_events {
-                let aev = StateResolution::get_or_load_event(room_id, aid, event_map)?;
+                let aev = fetch_event(aid)
+                    .ok_or_else(|| Error::NotFound(format!("Failed to find {}", aid)))?;
                 if is_type_and_key(&aev, EventType::RoomPowerLevels, "") {
                     event = Some(aev);
                     break;
@@ -587,23 +604,22 @@ impl StateResolution {
         Ok(0)
     }
 
-    fn add_event_and_auth_chain_to_graph<E: Event>(
-        room_id: &RoomId,
-        graph: &mut BTreeMap<EventId, BTreeSet<EventId>>,
+    fn add_event_and_auth_chain_to_graph<E, F>(
+        graph: &mut HashMap<EventId, HashSet<EventId>>,
         event_id: &EventId,
-        event_map: &mut EventMap<Arc<E>>,
-        auth_diff: &BTreeSet<EventId>,
-    ) {
+        auth_diff: &HashSet<EventId>,
+        fetch_event: F,
+    ) where
+        E: Event,
+        F: Fn(&EventId) -> Option<Arc<E>>,
+    {
         let mut state = vec![event_id.clone()];
         while !state.is_empty() {
             // We just checked if it was empty so unwrap is fine
             let eid = state.pop().unwrap();
-            graph.entry(eid.clone()).or_insert(btreeset![]);
+            graph.entry(eid.clone()).or_insert(hashset![]);
             // Prefer the store to event as the store filters dedups the events
-            for aid in &StateResolution::get_or_load_event(room_id, &eid, event_map)
-                .map(|ev| ev.auth_events())
-                .unwrap_or_default()
-            {
+            for aid in &fetch_event(&eid).map(|ev| ev.auth_events()).unwrap_or_default() {
                 if auth_diff.contains(aid) {
                     if !graph.contains_key(aid) {
                         state.push(aid.clone());
@@ -615,22 +631,14 @@ impl StateResolution {
             }
         }
     }
-
-    /// Uses the `event_map` to return the full PDU or fails.
-    fn get_or_load_event<E: Event>(
-        _room_id: &RoomId,
-        ev_id: &EventId,
-        event_map: &EventMap<Arc<E>>,
-    ) -> Result<Arc<E>> {
-        event_map.get(ev_id).map_or_else(
-            || Err(Error::NotFound(format!("EventId: {:?} not found", ev_id))),
-            |e| Ok(Arc::clone(e)),
-        )
-    }
 }
 
-pub fn is_power_event_id<E: Event>(event_id: &EventId, event_map: &EventMap<Arc<E>>) -> bool {
-    match event_map.get(event_id) {
+pub fn is_power_event_id<E, F>(event_id: &EventId, fetch: F) -> bool
+where
+    E: Event,
+    F: Fn(&EventId) -> Option<Arc<E>>,
+{
+    match fetch(event_id).as_ref() {
         Some(state) => is_power_event(state),
         _ => false,
     }
