@@ -1,21 +1,24 @@
-use std::collections::{BTreeMap, BTreeSet};
+use std::{
+    collections::{HashMap, HashSet},
+    sync::Arc,
+};
 
 use rand::seq::SliceRandom;
 use ruma_events::EventType;
 use ruma_state_res::{is_power_event, room_version::RoomVersion, StateMap, StateResolution};
 
 mod utils;
-use utils::{room_id, INITIAL_EVENTS};
+use utils::INITIAL_EVENTS;
 
 fn test_event_sort() {
-    let mut events = INITIAL_EVENTS();
+    let events = INITIAL_EVENTS();
 
     let event_map = events
         .values()
         .map(|ev| ((ev.kind(), ev.state_key()), ev.clone()))
         .collect::<StateMap<_>>();
 
-    let auth_chain = BTreeSet::new();
+    let auth_chain = HashSet::new();
 
     let power_events = event_map
         .values()
@@ -23,24 +26,17 @@ fn test_event_sort() {
         .map(|pdu| pdu.event_id().clone())
         .collect::<Vec<_>>();
 
-    // This is a TODO in conduit
-    // TODO these events are not guaranteed to be sorted but they are resolved, do
-    // we need the auth_chain
-    let sorted_power_events = StateResolution::reverse_topological_power_sort(
-        &room_id(),
-        &power_events,
-        &mut events,
-        &auth_chain,
-    );
+    let sorted_power_events =
+        StateResolution::reverse_topological_power_sort(&power_events, &auth_chain, |id| {
+            events.get(id).map(Arc::clone)
+        })
+        .unwrap();
 
-    // This is a TODO in conduit
-    // TODO we may be able to skip this since they are resolved according to spec
     let resolved_power = StateResolution::iterative_auth_check(
-        &room_id(),
         &RoomVersion::version_6(),
         &sorted_power_events,
-        &BTreeMap::new(), // unconflicted events
-        &mut events,
+        &HashMap::new(), // unconflicted events
+        |id| events.get(id).map(Arc::clone),
     )
     .expect("iterative auth check failed on resolved events");
 
@@ -49,10 +45,12 @@ fn test_event_sort() {
 
     events_to_sort.shuffle(&mut rand::thread_rng());
 
-    let power_level = resolved_power.get(&(EventType::RoomPowerLevels, "".to_string()));
+    let power_level = resolved_power.get(&(EventType::RoomPowerLevels, "".to_owned()));
 
-    let sorted_event_ids =
-        StateResolution::mainline_sort(&room_id(), &events_to_sort, power_level, &mut events);
+    let sorted_event_ids = StateResolution::mainline_sort(&events_to_sort, power_level, |id| {
+        events.get(id).map(Arc::clone)
+    })
+    .unwrap();
 
     assert_eq!(
         vec![
