@@ -1,7 +1,7 @@
 use std::sync::Arc;
 
 use js_int::uint;
-use maplit::{btreemap, btreeset};
+use maplit::{hashmap, hashset};
 use ruma_common::MilliSecondsSinceUnixEpoch;
 use ruma_events::{room::join_rules::JoinRule, EventType};
 use ruma_identifiers::{EventId, RoomVersionId};
@@ -247,14 +247,14 @@ fn test_event_map_none() {
     let _ = LOGGER
         .call_once(|| tracer::fmt().with_env_filter(tracer::EnvFilter::from_default_env()).init());
 
-    let mut store = TestStore::<StateEvent>(btreemap! {});
+    let mut store = TestStore::<StateEvent>(hashmap! {});
 
     // build up the DAG
     let (state_at_bob, state_at_charlie, expected) = store.set_up();
 
-    let mut ev_map: EventMap<Arc<StateEvent>> = store.0.clone();
+    let ev_map: EventMap<Arc<StateEvent>> = store.0.clone();
     let state_sets = vec![state_at_bob, state_at_charlie];
-    let resolved = match StateResolution::resolve::<StateEvent>(
+    let resolved = match StateResolution::resolve::<StateEvent, _>(
         &room_id(),
         &RoomVersionId::Version2,
         &state_sets,
@@ -266,7 +266,7 @@ fn test_event_map_none() {
                     .unwrap()
             })
             .collect(),
-        &mut ev_map,
+        |id| ev_map.get(id).map(Arc::clone),
     ) {
         Ok(state) => state,
         Err(e) => panic!("{}", e),
@@ -277,17 +277,18 @@ fn test_event_map_none() {
 
 #[test]
 fn test_lexicographical_sort() {
-    let graph = btreemap! {
-        event_id("l") => btreeset![event_id("o")],
-        event_id("m") => btreeset![event_id("n"), event_id("o")],
-        event_id("n") => btreeset![event_id("o")],
-        event_id("o") => btreeset![], // "o" has zero outgoing edges but 4 incoming edges
-        event_id("p") => btreeset![event_id("o")],
+    let graph = hashmap! {
+        event_id("l") => hashset![event_id("o")],
+        event_id("m") => hashset![event_id("n"), event_id("o")],
+        event_id("n") => hashset![event_id("o")],
+        event_id("o") => hashset![], // "o" has zero outgoing edges but 4 incoming edges
+        event_id("p") => hashset![event_id("o")],
     };
 
     let res = StateResolution::lexicographical_topological_sort(&graph, |id| {
-        (0, MilliSecondsSinceUnixEpoch(uint!(0)), id.clone())
-    });
+        Ok((0, MilliSecondsSinceUnixEpoch(uint!(0)), id.clone()))
+    })
+    .unwrap();
 
     assert_eq!(
         vec!["o", "l", "n", "m", "p"],
